@@ -3,31 +3,26 @@ use swc_atoms::JsWord;
 use swc_common::Spanned;
 use swc_ecma_ast::*;
 use swc_ecma_utils::quote_str;
-use swc_ecma_visit::{noop_fold_type, Fold, FoldWith};
+use swc_ecma_visit::{as_folder, noop_visit_mut_type, Fold, VisitMut, VisitMutWith};
 
-pub fn duplicate_keys() -> impl Fold {
-    DuplicateKeys
+pub fn duplicate_keys() -> impl Fold + VisitMut {
+    as_folder(DuplicateKeys)
 }
 
 struct DuplicateKeys;
 
-impl Fold for DuplicateKeys {
-    noop_fold_type!();
+impl VisitMut for DuplicateKeys {
+    noop_visit_mut_type!();
 
-    fn fold_expr(&mut self, expr: Expr) -> Expr {
-        let expr = expr.fold_children_with(self);
+    fn visit_mut_expr(&mut self, expr: &mut Expr) {
+        expr.visit_mut_children_with(self);
 
         match expr {
-            Expr::Object(ObjectLit { span, props }) => {
+            Expr::Object(ObjectLit { props, .. }) => {
                 let mut folder = PropFolder::default();
-
-                ObjectLit {
-                    span,
-                    props: props.fold_with(&mut folder),
-                }
-                .into()
+                props.visit_mut_with(&mut folder);
             }
-            _ => expr,
+            _ => {}
         }
     }
 }
@@ -38,45 +33,42 @@ struct PropFolder {
     setter_props: FxHashSet<JsWord>,
 }
 
-impl Fold for PropFolder {
-    noop_fold_type!();
+impl VisitMut for PropFolder {
+    noop_visit_mut_type!();
 
-    fn fold_expr(&mut self, node: Expr) -> Expr {
-        node
-    }
+    /// Noop
+    fn visit_mut_expr(&mut self, _: &mut Expr) {}
 
-    fn fold_prop(&mut self, prop: Prop) -> Prop {
+    fn visit_mut_prop(&mut self, prop: &mut Prop) {
         match prop {
             Prop::Shorthand(ident) => {
                 //
                 if !self.getter_props.insert(ident.sym.clone())
                     || !self.setter_props.insert(ident.sym.clone())
                 {
-                    Prop::KeyValue(KeyValueProp {
+                    *prop = Prop::KeyValue(KeyValueProp {
                         key: PropName::Computed(ComputedPropName {
                             span: ident.span,
                             expr: Box::new(Expr::Lit(Lit::Str(quote_str!(ident.sym.clone())))),
                         }),
-                        value: Box::new(Expr::Ident(ident)),
+                        value: Box::new(Expr::Ident(ident.clone())),
                     })
-                } else {
-                    Prop::Shorthand(ident)
                 }
             }
 
             Prop::Assign(..) => unreachable!("assign property in object literal is invalid"),
 
-            Prop::Getter(..) => prop.fold_children_with(&mut PropNameFolder {
+            Prop::Getter(..) => prop.visit_mut_children_with(&mut PropNameFolder {
                 props: &mut self.getter_props,
             }),
-            Prop::Setter(..) => prop.fold_children_with(&mut PropNameFolder {
+            Prop::Setter(..) => prop.visit_mut_children_with(&mut PropNameFolder {
                 props: &mut self.setter_props,
             }),
             _ => {
-                let prop = prop.fold_children_with(&mut PropNameFolder {
+                prop.visit_mut_children_with(&mut PropNameFolder {
                     props: &mut self.getter_props,
                 });
-                prop.fold_children_with(&mut PropNameFolder {
+                prop.visit_mut_children_with(&mut PropNameFolder {
                     props: &mut self.setter_props,
                 })
             }
@@ -87,53 +79,47 @@ impl Fold for PropFolder {
 struct PropNameFolder<'a> {
     props: &'a mut FxHashSet<JsWord>,
 }
-impl<'a> Fold for PropNameFolder<'a> {
-    noop_fold_type!();
 
-    fn fold_expr(&mut self, node: Expr) -> Expr {
-        node
-    }
+impl<'a> VisitMut for PropNameFolder<'a> {
+    noop_visit_mut_type!();
 
-    fn fold_prop_name(&mut self, name: PropName) -> PropName {
+    fn visit_mut_expr(&mut self, _: &mut Expr) {}
+
+    fn visit_mut_prop_name(&mut self, name: &mut PropName) {
         let span = name.span();
 
         match name {
             PropName::Ident(ident) => {
                 if !self.props.insert(ident.sym.clone()) {
-                    PropName::Computed(ComputedPropName {
+                    *name = PropName::Computed(ComputedPropName {
                         span,
                         expr: Box::new(Expr::Lit(Lit::Str(Str {
                             span,
-                            value: ident.sym,
+                            value: ident.sym.clone(),
                             has_escape: false,
                             kind: Default::default(),
                         }))),
                     })
-                } else {
-                    PropName::Ident(ident)
                 }
             }
             PropName::Str(s) => {
                 if !self.props.insert(s.value.clone()) {
-                    PropName::Computed(ComputedPropName {
+                    *name = PropName::Computed(ComputedPropName {
                         span: s.span,
-                        expr: Box::new(Expr::Lit(Lit::Str(s))),
+                        expr: Box::new(Expr::Lit(Lit::Str(s.clone()))),
                     })
-                } else {
-                    PropName::Str(s)
                 }
             }
-            PropName::Computed(ComputedPropName { span, expr }) => {
+            PropName::Computed(ComputedPropName { expr, .. }) => {
                 // Computed property might collide
-                match *expr {
+                match &**expr {
                     Expr::Lit(Lit::Str(Str { ref value, .. })) => {
                         self.props.insert(value.clone());
                     }
                     _ => {}
                 }
-                PropName::Computed(ComputedPropName { span, expr })
             }
-            _ => name,
+            _ => {}
         }
     }
 }
