@@ -1,7 +1,8 @@
+use swc_common::util::take::Take;
 use swc_ecma_ast::*;
 use swc_ecma_transforms_base::ext::PatOrExprExt;
 use swc_ecma_utils::{private_ident, UsageFinder};
-use swc_ecma_visit::{noop_fold_type, Fold, FoldWith};
+use swc_ecma_visit::{as_folder, noop_visit_mut_type, Fold, VisitMut, VisitMutWith};
 
 /// `@babel/plugin-transform-function-name`
 ///
@@ -20,8 +21,8 @@ use swc_ecma_visit::{noop_fold_type, Fold, FoldWith};
 /// }
 /// var Foo = (class Foo {});
 /// ```
-pub fn function_name() -> impl Fold {
-    FnName
+pub fn function_name() -> impl 'static + Copy + Fold + VisitMut {
+    as_folder(FnName)
 }
 
 #[derive(Clone, Copy)]
@@ -40,14 +41,14 @@ fn prepare(i: Ident) -> Ident {
     i
 }
 
-impl Fold for FnName {
-    noop_fold_type!();
+impl VisitMut for FnName {
+    noop_visit_mut_type!();
 
-    fn fold_assign_expr(&mut self, expr: AssignExpr) -> AssignExpr {
-        let mut expr = expr.fold_children_with(self);
+    fn visit_mut_assign_expr(&mut self, expr: &mut AssignExpr) {
+        expr.visit_mut_children_with(self);
 
         if expr.op != op!("=") {
-            return expr;
+            return;
         }
 
         if let Some(ident) = expr.left.as_ident_mut() {
@@ -55,81 +56,67 @@ impl Fold for FnName {
                 name: Some(ident.clone()),
             };
 
-            let right = expr.right.fold_with(&mut folder);
+            expr.right.visit_mut_with(&mut folder);
 
-            return AssignExpr { right, ..expr };
+            return;
         }
-
-        expr
     }
 
-    fn fold_key_value_prop(&mut self, p: KeyValueProp) -> KeyValueProp {
-        let mut p = p.fold_children_with(self);
+    fn visit_mut_key_value_prop(&mut self, p: &mut KeyValueProp) {
+        p.visit_mut_children_with(self);
 
-        p.value = match *p.value {
+        match &mut *p.value {
             Expr::Fn(expr @ FnExpr { ident: None, .. }) => {
                 //
-                if let PropName::Ident(ref i) = p.key {
+                p.value = if let PropName::Ident(ref i) = p.key {
                     Box::new(Expr::Fn(FnExpr {
                         ident: Some(prepare(i.clone())),
-                        ..expr
+                        ..expr.take()
                     }))
                 } else {
-                    Box::new(Expr::Fn(expr))
-                }
+                    Box::new(Expr::Fn(expr.take()))
+                };
             }
-            _ => p.value,
+            _ => {}
         };
-
-        p
     }
 
-    fn fold_var_declarator(&mut self, decl: VarDeclarator) -> VarDeclarator {
-        let mut decl = decl.fold_children_with(self);
+    fn visit_mut_var_declarator(&mut self, decl: &mut VarDeclarator) {
+        decl.visit_mut_children_with(self);
 
         match decl.name {
             Pat::Ident(ref mut ident) => {
                 let mut folder = Renamer {
                     name: Some(prepare(ident.id.clone())),
                 };
-                let init = decl.init.fold_with(&mut folder);
-
-                VarDeclarator { init, ..decl }
+                decl.init.visit_mut_with(&mut folder);
             }
-            _ => decl,
+            _ => {}
         }
     }
 }
 
 macro_rules! impl_for {
     ($name:ident, $T:tt) => {
-        fn $name(&mut self, node: $T) -> $T {
+        fn $name(&mut self, node: &mut $T) {
             match node.ident {
-                Some(..) => return node,
+                Some(..) => return,
                 None => {
                     //
                     let name = match self.name.take() {
                         None => {
-                            return $T {
-                                ident: None,
-                                ..node
-                            };
+                            node.ident = None;
+                            return;
                         }
                         Some(name) => name,
                     };
                     // If function's body references the name of variable, we just skip the
                     // function
-                    if UsageFinder::find(&name, &node) {
+                    if UsageFinder::find(&name, &*node) {
                         // self.name = Some(name);
-                        $T {
-                            ident: None,
-                            ..node
-                        }
+                        node.ident = None;
                     } else {
-                        $T {
-                            ident: Some(name),
-                            ..node
-                        }
+                        node.ident = Some(name);
                     }
                 }
             }
@@ -140,22 +127,20 @@ macro_rules! impl_for {
 macro_rules! noop {
     ($name:ident, $T:tt) => {
         /// Don't recurse.
-        fn $name(&mut self, node: $T) -> $T {
-            node
-        }
+        fn $name(&mut self, _: &mut $T) {}
     };
 }
 
-impl Fold for Renamer {
-    noop_fold_type!();
+impl VisitMut for Renamer {
+    noop_visit_mut_type!();
 
-    impl_for!(fold_fn_expr, FnExpr);
-    impl_for!(fold_class_expr, ClassExpr);
+    impl_for!(visit_mut_fn_expr, FnExpr);
+    impl_for!(visit_mut_class_expr, ClassExpr);
 
-    noop!(fold_object_lit, ObjectLit);
-    noop!(fold_array_lit, ArrayLit);
-    noop!(fold_call_expr, CallExpr);
-    noop!(fold_new_expr, NewExpr);
-    noop!(fold_bin_expr, BinExpr);
-    noop!(fold_unary_expr, UnaryExpr);
+    noop!(visit_mut_object_lit, ObjectLit);
+    noop!(visit_mut_array_lit, ArrayLit);
+    noop!(visit_mut_call_expr, CallExpr);
+    noop!(visit_mut_new_expr, NewExpr);
+    noop!(visit_mut_bin_expr, BinExpr);
+    noop!(visit_mut_unary_expr, UnaryExpr);
 }
