@@ -5,7 +5,7 @@ use swc_ecma_ast::*;
 use swc_ecma_utils::{find_ids, private_ident};
 use swc_ecma_visit::{noop_fold_type, Fold, FoldWith};
 
-pub(super) type Vars = SmallVec<[Ident; 32]>;
+pub(super) type Vars = SmallVec<[Ident; 12]>;
 
 pub(super) fn hoist<T>(node: T) -> (T, Hoister)
 where
@@ -138,7 +138,10 @@ impl Fold for Hoister {
 
             VarDeclOrPat::VarDecl(mut var) => {
                 if var.decls.len() == 1 && var.decls[0].init.is_none() {
-                    return var.decls.remove(0).name.into();
+                    let pat = var.decls.remove(0).name;
+                    self.vars.extend(find_ids(&pat));
+
+                    return pat.into();
                 }
 
                 var.into()
@@ -154,4 +157,20 @@ impl Fold for Hoister {
             _ => var.fold_children_with(self),
         }
     }
+
+    fn fold_opt_var_decl_or_expr(&mut self, v: Option<VarDeclOrExpr>) -> Option<VarDeclOrExpr> {
+        let v = v.fold_children_with(self);
+
+        match &v {
+            Some(VarDeclOrExpr::Expr(e)) => {
+                if e.is_invalid() {
+                    return None;
+                }
+            }
+
+            _ => {}
+        }
+
+        v
+    }
 }
